  include 'blocks.f90'
  include 'formats.f90'
  include 'constants.f90'
  program PF_p
    use blocks
    use formats
    implicit none
      
    real*8    :: p, dp, RL, RL_0, RL_eq, RL_0_eq, fdist, error

    integer*4 :: info, ip, ip_fail, iter

    logical*4 :: flag

    real*8, allocatable, dimension(:) :: z, y, F

    external check_pars, update_z, make_F, fdist

    Job = 'PF_p'

401 format(a)
402 format(a,i0)

    write(6,f1) '=========================================='
    write(6,f1) '=========================================='
    write(6,f1) '=========================================='
    call check_pars(info)
    write(6,f1) '******************************************'
    if(info .eq. 0) then
       write(*,401) 'PF_p: Parameters checked for sanity'
    else if(info .gt. 0) then
       write(*,402) 'PF_p: There were errors on input, info=', info
       stop
    else 
       write(*,402) 'PF_p: There were warnings on input, info=', info
    end if

    dp = (p_max - p_min)/dfloat(Np-1)
    if(dp .lt. small) then
       write(*,*) 'PF_p: dp is too small, exiting; dp=', dp
       stop
    end if

    write(6,f1) 'PF_p computes L as functions of p at fixed kappa and e=0'
    write(6,f1) 'File "L0.dat" : p, L(p)/L(0) (no electrostatic interaction)'
    write(6,f1) 'File  "L.dat" : p, L(p)/L(0) (with electrostatic interaction)'
    write(6,f1) '... where'
    write(6,f3) '[from parameter p_min] -- p_min: ', sngl(p_min)
    write(6,f3) '[from parameter p_max] -- p_max: ', sngl(p_max)
    write(6,f3) '[from parameter kappa] -- kappa: ', sngl(kappa)
    write(6,f1) '******************************************'

    allocate(z(1:N))
    allocate(y(1:N))
    allocate(F(1:N))

    open(unit=71, file='L0.dat', status='unknown') ! No   electrostatic forces
    open(unit=72, file='L.dat' , status='unknown') ! With electrostatic forces

    flag = .false.

    ip_loop: do ip=1,Np
       p = p_min + dp*dfloat(ip-1)

       F = rz; F(1) = p; F(N) = -p
       call update_z(F, z, error)
       if(error .gt. small) then
          write(6,*) 'PF_p: Error in update_z at Point A:', error
          flag = .true.
          ip_fail = ip
          exit ip_loop
       end if

       RL_0 = z(N) - z(1)
       if(p .lt. tiny) then
          RL_0_eq = RL_0
       else
          write(71,*) sngl(p), sngl(RL_0/RL_0_eq)
       end if

       iter = 0
       do_loop: do
          iter = iter + 1
          if(iter .gt. iter_max) then
             write(6,*) 'PF_p: maximum number of iterations has been reached'
             flag = .true.
             ip_fail = ip
             exit ip_loop
          end if
          call make_F(z,F)
          F(1) = F(1) + p; F(N) = F(N) - p
          y    = z
          call update_z(F, z, error)
          if(error .gt. small) then
             write(6,*) 'PF_p: Error in update_z at Point B:', error
             flag = .true.
             ip_fail = ip
             exit ip_loop
          end if
          if(fdist(z,y) .le. th1) exit do_loop
       end do do_loop

       RL = z(N) - z(1)
       if(p .lt. tiny) then
          RL_eq = RL
          write(72,*) ' 0.0            1.0'
       else
          write(72,*) sngl(p), sngl(RL/RL_eq)
       end if
      
400    format(a1, a18, i10, a4, i10)
       if(Vocal) then
          write(6,*) sngl(p), sngl(RL), iter
       else
          write(6, 400, advance='NO') CC, '------ Complete : ', ip , ' of ', Np
          flush(6)
       end if

    end do ip_loop

    close(unit=71); close(unit=72)

    write(*,*)
      
    if(flag) then
       write(6,f4) 'PF_p: all data points up to ip=', ip_fail, ' were computed successfully'
    else
       write(6,f1) 'PF_p: all data points computed successfully'
    end if

    stop
  end program PF_p
