subroutine make_dsum(Nthr, nmin, nmax, L, h, Ss, Sd)
  use omp_lib
  implicit none
  integer*4, intent(in)  :: L, Nthr, nmin(0:Nthr-1), nmax(0:Nthr-1)
  real*8,    intent(in)  :: h
  real*8,    intent(out) :: Ss, Sd

  real*8                 :: h20, h21, arg_a, arg_b, arg_2
  real*8                 :: sum_a, sum_b, sum_2, pum_a, pum_b, pum_2
  integer*8              :: n, m, a2, b2, c2, d2
  integer*4              :: id

  h20 = h*h 
  h21 = h20 + 1.0d0

  sum_2 = 0.0d0
  sum_a = 0.0d0

  !$OMP PARALLEL PRIVATE(pum_a, pum_2, n, m, a2, b2, c2, d2, arg_a, arg_2) SHARED(sum_a, sum_2, L, h20, h21, h, nmin, nmax)

  pum_a = 0.0d0; pum_2 = 0.0d0; 

  id = omp_get_thread_num()

  do n=nmin(id),nmax(id)
     a2 = n*n
     c2 = a2 - n
     do m=1,n-1
        b2 = m*m
        d2 = b2 - m
        arg_a = dfloat(2*(a2 + b2)) + h20
        arg_2 = dfloat(2*(c2 + d2)) + h21
        pum_a = pum_a + h / (arg_a*dsqrt(arg_a))
        pum_2 = pum_2 + h / (arg_2*dsqrt(arg_2))
     end do
  end do

  !$OMP CRITICAL
  sum_a = sum_a + pum_a; sum_2 = sum_2 + pum_2 
  !$OMP END CRITICAL

  !$OMP END PARALLEL

  sum_a = sum_a * 2.0d0
  sum_2 = sum_2 * 2.0d0
  sum_b = 0.0d0

  !$OMP PARALLEL PRIVATE(pum_a, pum_b, pum_2, n, a2, c2, arg_a, arg_b, arg_2) SHARED(sum_a, sum_b, sum_2, L, h20, h21, h)
  pum_a = 0.0d0
  pum_b = 0.0d0
  pum_2 = 0.0d0; 

  !$OMP DO
  do n=1,L
     a2 = n*n
     c2 = a2 - n
     arg_a = dfloat(4*a2) + h20
     arg_b = dfloat(2*a2) + h20
     arg_2 = dfloat(4*c2) + h21
     pum_a = pum_a + h / (arg_a*dsqrt(arg_a))
     pum_b = pum_b + h / (arg_b*dsqrt(arg_b))
     pum_2 = pum_2 + h / (arg_2*dsqrt(arg_2))
  end do
  !$OMP END DO

  !$OMP CRITICAL
  sum_a = sum_a + pum_a
  sum_b = sum_b + pum_b
  sum_2 = sum_2 + pum_2 
  !$OMP END CRITICAL

  !$OMP END PARALLEL

  Ss = 4.0d0*(sum_a + sum_b) + 1.0d0/h20
  Sd = 4.0d0*sum_2

  return
end subroutine make_dsum
