subroutine make_qsum(Nthr, nmin, nmax, L, h, Ss, Sd)
  use omp_lib
  implicit none
  integer*4, intent(in)  :: L, Nthr, nmin(0:Nthr-1), nmax(0:Nthr-1)
  real*8,    intent(in)  :: h
  real*8,    intent(out) :: Ss, Sd

  real*16                :: h20, h21, hq, arg_a, arg_b, arg_2
  real*16                :: sum_1, sum_a, sum_b, sum_2, pum_a, pum_b, pum_2
  integer*8              :: n, m, a2, b2, c2, d2
  integer*4              :: id

  hq  = qextd(h)
  h20 = hq*hq 
  h21 = h20 + 1.0q0

  sum_2 = 0.0q0
  sum_a = 0.0q0

  !$OMP PARALLEL PRIVATE(pum_a, pum_2, n, m, a2, b2, c2, d2, arg_a, arg_2) SHARED(sum_a, sum_2, L, h20, h21, hq, nmin, nmax)

  pum_a = 0.0q0; pum_2 = 0.0q0; 

  id = omp_get_thread_num()

  do n=nmin(id),nmax(id)
     a2 = n*n
     c2 = a2 - n
     do m=1,n-1
        b2 = m*m
        d2 = b2 - m
        arg_a = qfloat(2*(a2 + b2)) + h20
        arg_2 = qfloat(2*(c2 + d2)) + h21
        pum_a = pum_a + hq / (arg_a*qsqrt(arg_a))
        pum_2 = pum_2 + hq / (arg_2*qsqrt(arg_2))
     end do
  end do

  !$OMP CRITICAL
  sum_a = sum_a + pum_a; sum_2 = sum_2 + pum_2 
  !$OMP END CRITICAL

  !$OMP END PARALLEL

  sum_a = sum_a * 2.0q0
  sum_2 = sum_2 * 2.0q0
  sum_b = 0.0q0

  !$OMP PARALLEL PRIVATE(pum_a, pum_b, pum_2, n, a2, c2, arg_a, arg_b, arg_2) SHARED(sum_a, sum_b, sum_2, L, h20, h21, hq)
  pum_a = 0.0q0
  pum_b = 0.0q0
  pum_2 = 0.0q0; 

  !$OMP DO
  do n=1,L
     a2 = n*n
     c2 = a2 - n
     arg_a = qfloat(4*a2) + h20
     arg_b = qfloat(2*a2) + h20
     arg_2 = qfloat(4*c2) + h21
     pum_a = pum_a + hq / (arg_a*qsqrt(arg_a))
     pum_b = pum_b + hq / (arg_b*qsqrt(arg_b))
     pum_2 = pum_2 + hq / (arg_2*qsqrt(arg_2))
  end do
  !$OMP END DO

  !$OMP CRITICAL
  sum_a = sum_a + pum_a
  sum_b = sum_b + pum_b
  sum_2 = sum_2 + pum_2 
  !$OMP END CRITICAL

  !$OMP END PARALLEL

  sum_1 = 4.0q0*(sum_a + sum_b) + 1.0q0/h20
  sum_2 = 4.0q0*sum_2

  Ss    = dble(sum_1)
  Sd    = dble(sum_2)

  return
end subroutine make_qsum
